@@ -2,6 +2,7 @@
  * Copyright (c) 2003 Riccardo Persichetti <riccardo.persichetti@tin.it>
  * Copyright (c) 2010 Florian Rivoal <frivoal@xfce.org>
  * Copyright (c) 2012 David Schneider <dnschneid@gmail.com>
+ * Copyright (c) 2014-2016 Landry Breuil <landry@xfce.org>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -52,29 +53,29 @@ static gchar *MONITOR_ROOT[] = { "SL_Cpu", "SL_Mem", "SL_Swap", "SL_Uptime" };
 
 static gchar *DEFAULT_TEXT[] = { "cpu", "mem", "swap" };
 static gchar *DEFAULT_COLOR[] = { "#0000c0", "#00c000", "#f0f000" };
+static gchar *DEFAULT_COMMAND_TEXT = "xfce4-taskmanager";
 
 #define UPDATE_TIMEOUT 250
 #define UPDATE_TIMEOUT_SECONDS 1
 
 #define BORDER 8
 
-/* check for new Xfce 4.10 panel features */
-#ifdef LIBXFCE4PANEL_CHECK_VERSION
-#if LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
-#define HAS_PANEL_49
-#endif
-#endif
-
 enum { CPU_MONITOR, MEM_MONITOR, SWAP_MONITOR };
 
 typedef struct
 {
     gboolean enabled;
     gboolean use_label;
-    GdkColor color;
+    GdkRGBA  color;
     gchar    *label_text;
 } t_monitor_options;
 
+typedef struct
+{
+    gboolean enabled;
+    gchar    *command_text;
+} t_command;
+
 typedef struct
 {
     GtkWidget  *box;
@@ -102,9 +103,11 @@ typedef struct
     XfcePanelPlugin   *plugin;
     GtkWidget         *ebox;
     GtkWidget         *box;
+    GtkWidget         *menu_item;
     guint             timeout, timeout_seconds;
     gboolean          use_timeout_seconds;
     guint             timeout_id;
+    t_command         command;
     t_monitor         *monitor[3];
     t_uptime_monitor  *uptime;
 #ifdef HAVE_UPOWER_GLIB
@@ -112,6 +115,23 @@ typedef struct
 #endif
 } t_global_monitor;
 
+static gboolean
+spawn_system_monitor(GtkWidget *w, t_global_monitor *global)
+{
+    // Spawn defined command; In-terminal: false, Startup-notify: false
+    return xfce_spawn_command_line_on_screen(gdk_screen_get_default(),
+                                             global->command.command_text,
+                                             FALSE, FALSE, NULL);
+}
+
+static gboolean
+click_event(GtkWidget *w, GdkEventButton *event, t_global_monitor *global)
+{
+    if(event->button == 1 && global->command.enabled && *(global->command.command_text))
+        return spawn_system_monitor(w, global);
+    return FALSE;
+}
+
 static gint
 update_monitors(t_global_monitor *global)
 {
@@ -211,27 +231,17 @@ monitor_update_orientation (XfcePanelPlugin  *plugin,
                             t_global_monitor *global)
 {
     gint count;
-    xfce_hvbox_set_orientation(XFCE_HVBOX(global->box), panel_orientation);
+    gtk_orientable_set_orientation(GTK_ORIENTABLE(global->box), panel_orientation);
     for(count = 0; count < 3; count++)
     {
-        xfce_hvbox_set_orientation(XFCE_HVBOX(global->monitor[count]->box), panel_orientation);
-#ifdef HAS_PANEL_49
+        gtk_orientable_set_orientation(GTK_ORIENTABLE(global->monitor[count]->box), panel_orientation);
         gtk_label_set_angle(GTK_LABEL(global->monitor[count]->label),
                             (orientation == GTK_ORIENTATION_HORIZONTAL) ? 0 : -90);
-#endif
-        if (panel_orientation == GTK_ORIENTATION_HORIZONTAL)
-        {
-            gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(global->monitor[count]->status), GTK_PROGRESS_BOTTOM_TO_TOP);
-        }
-        else
-        {
-            gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(global->monitor[count]->status), GTK_PROGRESS_LEFT_TO_RIGHT);
-        }
+        gtk_progress_bar_set_inverted (GTK_PROGRESS_BAR(global->monitor[count]->status), (panel_orientation == GTK_ORIENTATION_HORIZONTAL));
+        gtk_orientable_set_orientation (GTK_ORIENTABLE(global->monitor[count]->status), !panel_orientation);
     }
-#ifdef HAS_PANEL_49
     gtk_label_set_angle(GTK_LABEL(global->uptime->label),
                         (orientation == GTK_ORIENTATION_HORIZONTAL) ? 0 : -90);
-#endif
 }
 
 static void
@@ -239,7 +249,7 @@ create_monitor (t_global_monitor *global)
 {
     gint count;
 
-    global->box = xfce_hvbox_new(xfce_panel_plugin_get_orientation(global->plugin), FALSE, 0);
+    global->box = gtk_box_new(xfce_panel_plugin_get_orientation(global->plugin), 0);
     gtk_widget_show(global->box);
 
     for(count = 0; count < 3; count++)
@@ -249,7 +259,7 @@ create_monitor (t_global_monitor *global)
 
         global->monitor[count]->status = GTK_WIDGET(gtk_progress_bar_new());
 
-        global->monitor[count]->box = xfce_hvbox_new(xfce_panel_plugin_get_orientation(global->plugin), FALSE, 0);
+        global->monitor[count]->box = gtk_box_new(xfce_panel_plugin_get_orientation(global->plugin), 0);
 
         gtk_box_pack_start(GTK_BOX(global->monitor[count]->box),
                            GTK_WIDGET(global->monitor[count]->label),
@@ -302,6 +312,8 @@ monitor_control_new(XfcePanelPlugin *plugin)
 {
     int count;
     t_global_monitor *global;
+    GtkWidget* image;
+    GtkWidget *label, *box;
     
     global = g_new(t_global_monitor, 1);
 #ifdef HAVE_UPOWER_GLIB
@@ -316,6 +328,19 @@ monitor_control_new(XfcePanelPlugin *plugin)
     gtk_widget_show(global->ebox);
     global->box = NULL;
 
+    global->command.enabled = FALSE;
+    global->command.command_text = g_strdup(DEFAULT_COMMAND_TEXT);
+
+    box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+    label = gtk_label_new_with_mnemonic (_("Run _System Monitor"));
+    global->menu_item = gtk_menu_item_new();
+    image = gtk_image_new_from_icon_name("utilities-system-monitor",
+                                                    GTK_ICON_SIZE_MENU);
+    gtk_container_add (GTK_CONTAINER (box), image);
+    gtk_container_add (GTK_CONTAINER (box), label);
+    gtk_container_add (GTK_CONTAINER (global->menu_item), box);
+    gtk_widget_show_all (global->menu_item);
+
     xfce_panel_plugin_add_action_widget (plugin, global->ebox);
     
     for(count = 0; count < 3; count++)
@@ -323,8 +348,8 @@ monitor_control_new(XfcePanelPlugin *plugin)
         global->monitor[count] = g_new(t_monitor, 1);
         global->monitor[count]->options.label_text =
             g_strdup(DEFAULT_TEXT[count]);
-        gdk_color_parse(DEFAULT_COLOR[count],
-                        &global->monitor[count]->options.color);
+        gdk_rgba_parse(&global->monitor[count]->options.color,
+                       DEFAULT_COLOR[count]);
 
         global->monitor[count]->options.use_label = TRUE;
         global->monitor[count]->options.enabled = TRUE;
@@ -357,6 +382,8 @@ monitor_free(XfcePanelPlugin *plugin, t_global_monitor *global)
     if (global->timeout_id)
         g_source_remove(global->timeout_id);
 
+    g_free(global->command.command_text);
+
     for(count = 0; count < 3; count++)
     {
         if (global->monitor[count]->options.label_text)
@@ -404,6 +431,10 @@ static void
 setup_monitor(t_global_monitor *global)
 {
     gint count;
+#if GTK_CHECK_VERSION (3, 16, 0)
+    GtkCssProvider *css_provider;
+    gchar * css;
+#endif
 
     gtk_widget_hide(GTK_WIDGET(global->uptime->ebox));
 
@@ -414,15 +445,32 @@ setup_monitor(t_global_monitor *global)
         gtk_label_set_text(GTK_LABEL(global->monitor[count]->label),
                            global->monitor[count]->options.label_text);
 
-        gtk_widget_modify_bg(GTK_WIDGET(global->monitor[count]->status),
+#if GTK_CHECK_VERSION (3, 16, 0)
+#if GTK_CHECK_VERSION (3, 20, 0)
+        css = g_strdup_printf("progressbar progress { background-color: %s; background-image: none; }",
+#else
+        css = g_strdup_printf(".progressbar { background-color: %s; background-image: none; }",
+#endif
+                                      gdk_rgba_to_string(&global->monitor[count]->options.color));
+        /* Setup Gtk style */
+        css_provider = gtk_css_provider_new ();
+        gtk_css_provider_load_from_data (css_provider, css, strlen(css), NULL);
+        gtk_style_context_add_provider (
+            GTK_STYLE_CONTEXT (gtk_widget_get_style_context (GTK_WIDGET (global->monitor[count]->status))),
+            GTK_STYLE_PROVIDER (css_provider),
+            GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+        g_free(css);
+#else
+        gtk_widget_override_background_color(GTK_WIDGET(global->monitor[count]->status),
                              GTK_STATE_PRELIGHT,
                              &global->monitor[count]->options.color);
-        gtk_widget_modify_bg(GTK_WIDGET(global->monitor[count]->status),
+        gtk_widget_override_background_color(GTK_WIDGET(global->monitor[count]->status),
                              GTK_STATE_SELECTED,
                              &global->monitor[count]->options.color);
-        gtk_widget_modify_base(GTK_WIDGET(global->monitor[count]->status),
+        gtk_widget_override_color(GTK_WIDGET(global->monitor[count]->status),
                                GTK_STATE_SELECTED,
                                &global->monitor[count]->options.color);
+#endif
 
         if(global->monitor[count]->options.enabled)
         {
@@ -472,6 +520,16 @@ monitor_read_config(XfcePanelPlugin *plugin, t_global_monitor *global)
                 rc, "Timeout_Seconds", global->timeout_seconds);
         global->use_timeout_seconds = xfce_rc_read_bool_entry (
                 rc, "Use_Timeout_Seconds", global->use_timeout_seconds);
+        global->command.enabled = xfce_rc_read_bool_entry (
+                rc, "Use_Click_Command", global->command.enabled);
+        value = xfce_rc_read_entry (
+                rc, "Click_Command", NULL);
+        if (value && *value)
+        {
+            if (global->command.command_text)
+                g_free(global->command.command_text);
+            global->command.command_text = g_strdup(value);
+        }
     }
 
     for(count = 0; count < 3; count++)
@@ -488,8 +546,8 @@ monitor_read_config(XfcePanelPlugin *plugin, t_global_monitor *global)
             
             if ((value = xfce_rc_read_entry (rc, "Color", NULL)))
             {
-                gdk_color_parse(value,
-                                &global->monitor[count]->options.color);
+                gdk_rgba_parse(&global->monitor[count]->options.color,
+                               value);
             }
             if ((value = xfce_rc_read_entry (rc, "Text", NULL)) && *value)
             {
@@ -514,7 +572,6 @@ monitor_read_config(XfcePanelPlugin *plugin, t_global_monitor *global)
 static void
 monitor_write_config(XfcePanelPlugin *plugin, t_global_monitor *global)
 {
-    char value[10];
     gint count;
     XfceRc *rc;
     char *file;
@@ -533,6 +590,8 @@ monitor_write_config(XfcePanelPlugin *plugin, t_global_monitor *global)
     xfce_rc_write_int_entry (rc, "Timeout_Seconds", global->timeout_seconds);
     xfce_rc_write_bool_entry (rc, "Use_Timeout_Seconds",
                               global->use_timeout_seconds);
+    xfce_rc_write_bool_entry (rc, "Use_Click_Command", global->command.enabled);
+    xfce_rc_write_entry (rc, "Click_Command", global->command.command_text);
 
     for(count = 0; count < 3; count++)
     {
@@ -544,12 +603,7 @@ monitor_write_config(XfcePanelPlugin *plugin, t_global_monitor *global)
         xfce_rc_write_bool_entry (rc, "Use_Label", 
                 global->monitor[count]->options.use_label);
 
-        g_snprintf(value, 8, "#%02X%02X%02X",
-                   (guint)global->monitor[count]->options.color.red >> 8,
-                   (guint)global->monitor[count]->options.color.green >> 8,
-                   (guint)global->monitor[count]->options.color.blue >> 8);
-
-        xfce_rc_write_entry (rc, "Color", value);
+        xfce_rc_write_entry (rc, "Color", gdk_rgba_to_string(&global->monitor[count]->options.color));
 
         xfce_rc_write_entry (rc, "Text", 
             global->monitor[count]->options.label_text ?
@@ -591,7 +645,6 @@ monitor_set_size(XfcePanelPlugin *plugin, int size, t_global_monitor *global)
 }
 
 
-#ifdef HAS_PANEL_49
 static void
 monitor_set_mode (XfcePanelPlugin *plugin, XfcePanelPluginMode mode,
                   t_global_monitor *global)
@@ -610,25 +663,17 @@ monitor_set_mode (XfcePanelPlugin *plugin, XfcePanelPluginMode mode,
   monitor_set_size (plugin, xfce_panel_plugin_get_size (plugin), global);
 }
 
-
-#else
-static void
-monitor_set_orientation (XfcePanelPlugin *plugin, GtkOrientation orientation,
-                         t_global_monitor *global)
-{
-  monitor_update_orientation (plugin, orientation, GTK_ORIENTATION_HORIZONTAL, global);
-  monitor_set_size (plugin, xfce_panel_plugin_get_size (plugin), global);
-}
-#endif
-
-
 #ifdef HAVE_UPOWER_GLIB
 static void
+#if UP_CHECK_VERSION(0, 99, 0)
+upower_changed_cb(UpClient *client, GParamSpec *pspec, t_global_monitor *global)
+#else /* UP_CHECK_VERSION < 0.99 */
 upower_changed_cb(UpClient *client, t_global_monitor *global)
+#endif /* UP_CHECK_VERSION */
 {
     setup_timer(global);
 }
-#endif
+#endif /* HAVE_UPOWER_GLIB */
 
 static void
 entry_changed_cb(GtkEntry *entry, t_global_monitor *global)
@@ -651,16 +696,36 @@ check_button_cb(GtkToggleButton *check_button, t_global_monitor *global)
     *boolvar = gtk_toggle_button_get_active(check_button);
     if (sensitive_widget)
         gtk_widget_set_sensitive(GTK_WIDGET(sensitive_widget), *boolvar);
-    if (oldstate != *boolvar)
+    if (boolvar == &(global->command.enabled)) {
+        gtk_widget_set_visible(global->menu_item, *boolvar);
+    } else if (oldstate != *boolvar)
+        setup_monitor(global);
+}
+
+static void
+switch_cb(GtkSwitch *check_button, gboolean state, t_global_monitor *global)
+{
+    gboolean oldstate;
+    gboolean* boolvar;
+    gpointer sensitive_widget;
+    boolvar = (gboolean*)g_object_get_data(G_OBJECT(check_button), "boolvar");
+    sensitive_widget = g_object_get_data(G_OBJECT(check_button), "sensitive_widget");
+    oldstate = *boolvar;
+    *boolvar = state;
+    if (sensitive_widget)
+        gtk_widget_set_sensitive(GTK_WIDGET(sensitive_widget), *boolvar);
+    if (boolvar == &(global->command.enabled)) {
+        gtk_widget_set_visible(global->menu_item, *boolvar);
+    } else if (oldstate != *boolvar)
         setup_monitor(global);
 }
 
 static void
 color_set_cb(GtkColorButton *color_button, t_global_monitor *global)
 {
-    GdkColor* colorvar;
-    colorvar = (GdkColor*)g_object_get_data(G_OBJECT(color_button), "colorvar");
-    gtk_color_button_get_color(color_button, colorvar);
+    GdkRGBA* colorvar;
+    colorvar = (GdkRGBA*)g_object_get_data(G_OBJECT(color_button), "colorvar");
+    gtk_color_chooser_get_rgba(GTK_COLOR_CHOOSER(color_button), colorvar);
     setup_monitor(global);
 }
 
@@ -691,117 +756,131 @@ change_timeout_seconds_cb(GtkSpinButton *spin, t_global_monitor *global)
 }
 #endif
 
-/* Create a new frame, optionally with a checkbox.
- * Set boolvar to NULL if you do not want a checkbox.
- * Returns the GtkTable inside the frame. */
-static GtkTable* new_frame(t_global_monitor *global, GtkBox *content,
-                           const gchar *title, guint rows, gboolean *boolvar)
-{
-    GtkWidget *frame, *table, *check, *label;
-    table = gtk_table_new (rows, 2, FALSE);
-    gtk_table_set_col_spacings (GTK_TABLE(table), 12);
-    gtk_table_set_row_spacings (GTK_TABLE(table), 6);
-    frame = xfce_gtk_frame_box_new_with_content (title, table);
-    gtk_box_pack_start_defaults (content, frame);
-    if (boolvar) {
-        check = gtk_check_button_new();
-        /* Move frame label into check button */
-        label = gtk_frame_get_label_widget (GTK_FRAME(frame));
-        g_object_ref (G_OBJECT(label));
-        gtk_container_remove (GTK_CONTAINER(frame), label);
-        gtk_container_add (GTK_CONTAINER(check), label);
-        g_object_unref (G_OBJECT(label));
-        /* Assign check to be the frame's label */
-        gtk_frame_set_label_widget (GTK_FRAME(frame), check);
-        /* Configure and set check button */
-        g_object_set_data (G_OBJECT(check), "sensitive_widget", table);
-        g_object_set_data (G_OBJECT(check), "boolvar", boolvar);
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check), *boolvar);
-        check_button_cb (GTK_TOGGLE_BUTTON(check), global);
-        g_signal_connect (G_OBJECT(check), "toggled",
-                          G_CALLBACK(check_button_cb), global);
-    }
-    return GTK_TABLE(table);
-}
-
-/* Creates a check box if boolvar is non-null, or a label if it is null.
+/* Creates a switch box if boolvar is non-null, and its label.
  * If it is a check box, it will control the sensitivity of target.
  * If it is a label, its mnemonic will point to target.
  * Returns the widget. */
 static GtkWidget *new_label_or_check_button(t_global_monitor *global,
+                                            GtkGrid *grid, guint row,
                                             const gchar *labeltext,
                                             gboolean *boolvar, GtkWidget *target)
 {
-    GtkWidget *label;
+    GtkWidget *label, *sw;
     if (boolvar) {
-        label = gtk_check_button_new_with_mnemonic (labeltext);
-        g_object_set_data (G_OBJECT(label), "sensitive_widget", target);
-        g_object_set_data (G_OBJECT(label), "boolvar", boolvar);
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(label), *boolvar);
-        check_button_cb (GTK_TOGGLE_BUTTON(label), global);
-        g_signal_connect (GTK_WIDGET(label), "toggled",
-                          G_CALLBACK(check_button_cb), global);
-    } else {
-        label = gtk_label_new_with_mnemonic (labeltext);
-        gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5f); \
-        gtk_label_set_mnemonic_widget(GTK_LABEL(label), target);
+        sw = gtk_switch_new();
+        g_object_set_data (G_OBJECT(sw), "sensitive_widget", target);
+        g_object_set_data (G_OBJECT(sw), "boolvar", boolvar);
+        gtk_switch_set_active (GTK_SWITCH(sw), *boolvar);
+        switch_cb (GTK_SWITCH(sw), *boolvar, global);
+        g_signal_connect (GTK_WIDGET(sw), "state-set",
+                          G_CALLBACK(switch_cb), global);
+        gtk_grid_attach(grid, sw, 2, row, 1, 1);
     }
+    label = gtk_label_new_with_mnemonic (labeltext);
+    gtk_widget_set_halign(label, GTK_ALIGN_END);
+    gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), target);
+    gtk_grid_attach(grid, label, 0, row, 1, 1);
     return label;
 }
 
-/* Adds an entry box to the table, optionally with a checkbox to enable it.
+/* Create a new monitor setting  with gtkswitch, and eventually a color button and a checkbox + entry */
+static void new_monitor_setting(t_global_monitor *global, GtkGrid *grid, int position,
+                           const gchar *title, gboolean *boolvar, GdkRGBA* colorvar,
+                           gboolean * use_label, gchar **labeltext)
+{
+    GtkWidget *subgrid, *sw, *label, *button, *check, *entry;
+    gchar *markup;
+
+    sw = gtk_switch_new();
+    g_object_set_data (G_OBJECT(sw), "boolvar", boolvar);
+    gtk_switch_set_active (GTK_SWITCH(sw), *boolvar);
+    switch_cb (GTK_SWITCH(sw), *boolvar, global);
+    g_signal_connect (GTK_WIDGET(sw), "state-set",
+                      G_CALLBACK(switch_cb), global);
+
+    markup = g_markup_printf_escaped ("<b>%s</b>", title);
+    label = gtk_label_new (markup);
+    gtk_widget_set_halign(label, GTK_ALIGN_START);
+    gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
+    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+    g_free (markup);
+    gtk_grid_attach(GTK_GRID(grid), label, 0, position, 2, 1);
+    gtk_grid_attach(GTK_GRID(grid), sw, 2, position, 1, 1);
+
+    if (colorvar == NULL)
+        return;
+    subgrid = gtk_grid_new ();
+    g_object_set_data (G_OBJECT(sw), "sensitive_widget", subgrid);
+    gtk_grid_attach(GTK_GRID(grid), subgrid, 0, position + 1, 2, 1);
+    gtk_grid_set_column_spacing (GTK_GRID(subgrid), 12);
+    gtk_grid_set_row_spacing (GTK_GRID(subgrid), 6);
+
+    button = gtk_color_button_new_with_rgba(colorvar);
+    g_object_set_data(G_OBJECT(button), "colorvar", colorvar);
+    g_signal_connect(G_OBJECT(button), "color-set",
+                 G_CALLBACK (color_set_cb), global);
+
+    label = gtk_label_new_with_mnemonic (_("Bar color:"));
+    gtk_widget_set_halign(label, GTK_ALIGN_END);
+    gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), button);
+
+    gtk_grid_attach(GTK_GRID(subgrid), label, 1, 1, 1, 1);
+    gtk_grid_attach(GTK_GRID(subgrid), button, 2, 1, 1, 1);
+
+    entry = gtk_entry_new ();
+    g_object_set_data (G_OBJECT(entry), "charvar", labeltext);
+    gtk_entry_set_text (GTK_ENTRY(entry), *labeltext);
+    g_signal_connect (G_OBJECT(entry), "changed",
+                      G_CALLBACK(entry_changed_cb), global);
+
+    label = gtk_label_new_with_mnemonic (_("Text to display:"));
+    gtk_widget_set_halign(label, GTK_ALIGN_END);
+    gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);
+
+    check = gtk_check_button_new();
+    g_object_set_data (G_OBJECT(check), "sensitive_widget", entry);
+    g_object_set_data (G_OBJECT(check), "boolvar", use_label);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check), *use_label);
+    check_button_cb (GTK_TOGGLE_BUTTON(check), global);
+    g_signal_connect (G_OBJECT(check), "toggled",
+                      G_CALLBACK(check_button_cb), global);
+
+    gtk_grid_attach(GTK_GRID(subgrid), check, 0, 0, 1, 1);
+    gtk_grid_attach(GTK_GRID(subgrid), label, 1, 0, 1, 1);
+    gtk_grid_attach(GTK_GRID(subgrid), entry, 2, 0, 1, 1);
+}
+
+/* Adds an entry box to the grid, optionally with a checkbox to enable it.
  * Set boolvar to NULL if you do not want a checkbox. */
-static void new_entry(t_global_monitor *global, GtkTable *table, guint row,
+static void new_entry(t_global_monitor *global, GtkGrid *grid, guint row,
                       const gchar *labeltext, gchar **charvar,
                       gboolean *boolvar)
 {
-    GtkWidget *label, *entry;
+    GtkWidget *entry;
     entry = gtk_entry_new ();
     g_object_set_data (G_OBJECT(entry), "charvar", charvar);
     gtk_entry_set_text (GTK_ENTRY(entry), *charvar);
     g_signal_connect (G_OBJECT(entry), "changed",
                       G_CALLBACK(entry_changed_cb), global);
-    label = new_label_or_check_button(global, labeltext, boolvar, entry);
-    gtk_table_attach_defaults(table, label,  0, 1, row, row+1);
-    gtk_table_attach_defaults(table, entry, 1, 2, row, row+1);
-}
-
-/* Adds a color button to the table, optionally with a checkbox to enable it.
- * Set boolvar to NULL if you do not want a checkbox. */
-static void new_color_button(t_global_monitor *global, GtkTable *table, guint row,
-                             const gchar *labeltext, GdkColor* colorvar,
-                             gboolean *boolvar)
-{
-    GtkWidget *label, *button;
-    button = gtk_color_button_new_with_color(colorvar);
-    g_object_set_data(G_OBJECT(button), "colorvar", colorvar);
-    g_signal_connect(G_OBJECT(button), "color-set",
-                     G_CALLBACK (color_set_cb), global);
-    label = new_label_or_check_button(global, labeltext, boolvar, button);
-    gtk_table_attach_defaults(table, label,  0, 1, row, row+1);
-    gtk_table_attach_defaults(table, button, 1, 2, row, row+1);
+    new_label_or_check_button(global, GTK_GRID(grid), row, labeltext, boolvar, entry);
+    gtk_grid_attach(grid, entry, 1, row, 1, 1);
 }
 
 /* Adds a new spin button, optionally with a checkbox to enable it.
  * Set boolvar to NULL if you do not want a checkbox. */
-static void new_spin_button(t_global_monitor *global, GtkTable *table, guint row,
-                            const gchar *labeltext, const gchar *units,
+static void new_spin_button(t_global_monitor *global, GtkGrid *grid, guint row,
+                            const gchar *labeltext,
                             gfloat value, gfloat min, gfloat max, gfloat step,
                             GCallback callback, gboolean* boolvar) {
-    GtkWidget *label, *button, *box;
-    /* Hbox for spin button + units */
-    box = gtk_hbox_new(TRUE, 2);
+    GtkWidget *button;
     button = gtk_spin_button_new_with_range (min, max, step);
     gtk_spin_button_set_value (GTK_SPIN_BUTTON (button), value);
     g_signal_connect (G_OBJECT (button), "value-changed", callback, global);
-    gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
-    label = gtk_label_new (units);
-    gtk_misc_set_alignment (GTK_MISC(label), 0, .5);
-    gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
-    /* Label/check button */
-    label = new_label_or_check_button(global, labeltext, boolvar, box);
-    gtk_table_attach_defaults(table, label,  0, 1, row, row+1);
-    gtk_table_attach_defaults(table, box, 1, 2, row, row+1);
+    new_label_or_check_button(global, grid, row, labeltext, boolvar, button);
+    gtk_grid_attach(grid, button, 1, row, 1, 1);
 }
 
 static void
@@ -809,7 +888,7 @@ monitor_create_options(XfcePanelPlugin *plugin, t_global_monitor *global)
 {
     GtkWidget           *dlg;
     GtkBox              *content;
-    GtkTable            *table;
+    GtkWidget           *grid;
     guint                count;
     t_monitor           *monitor;
     static const gchar *FRAME_TEXT[] = {
@@ -823,9 +902,8 @@ monitor_create_options(XfcePanelPlugin *plugin, t_global_monitor *global)
     
     dlg = xfce_titled_dialog_new_with_buttons (_("System Load Monitor"), 
                      GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
-                                               GTK_DIALOG_DESTROY_WITH_PARENT |
-                                               GTK_DIALOG_NO_SEPARATOR,
-                                               GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
+                                               GTK_DIALOG_DESTROY_WITH_PARENT,
+                                               "gtk-close", GTK_RESPONSE_OK,
                                                NULL);
     
     g_signal_connect (G_OBJECT (dlg), "response",
@@ -836,37 +914,41 @@ monitor_create_options(XfcePanelPlugin *plugin, t_global_monitor *global)
 
     content = GTK_BOX(gtk_dialog_get_content_area (GTK_DIALOG(dlg)));
 
-    table = new_frame(global, content, _("General"), 2, NULL);
-    new_spin_button(global, table, 0,
-            _("Update interval:"), _("s"),
+    grid = gtk_grid_new();
+    gtk_grid_set_column_spacing (GTK_GRID(grid), 12);
+    gtk_grid_set_row_spacing (GTK_GRID(grid), 6);
+    gtk_container_set_border_width(GTK_CONTAINER(grid), 6);
+    gtk_box_pack_start (content, grid, TRUE, TRUE, 0);
+    new_spin_button(global, GTK_GRID(grid), 0,
+            _("Update interval:"),
             (gfloat)global->timeout/1000.0, 0.100, 10.000, .050,
             G_CALLBACK(change_timeout_cb), NULL);
 #ifdef HAVE_UPOWER_GLIB
-    new_spin_button(global, table, 1,
-            _("Power-saving interval:"), _("s"),
+    new_spin_button(global, GTK_GRID(grid), 1,
+            _("Power-saving interval:"),
             (gfloat)global->timeout_seconds, 1, 10, 1,
             G_CALLBACK(change_timeout_seconds_cb),
             &global->use_timeout_seconds);
 #endif
-    
+    new_entry(global, GTK_GRID(grid), 2,
+              _("System monitor:"),
+              &global->command.command_text, &global->command.enabled);
     for(count = 0; count < 3; count++)
     {
         monitor = global->monitor[count];
 
-        table = new_frame(global, content,
-                          _(FRAME_TEXT[count]), 2, &monitor->options.enabled);
-
-        new_entry(global, table, 0,
-                  _("Text to display:"), &monitor->options.label_text,
-                  &monitor->options.use_label);
-
-        new_color_button(global, table, 1,
-                         _("Bar color:"), &monitor->options.color, NULL);
+        new_monitor_setting(global, GTK_GRID(grid), 3 + 2 * count,
+                           _(FRAME_TEXT[count]),
+                           &monitor->options.enabled,
+                           &monitor->options.color,
+                           &monitor->options.use_label,
+                           &monitor->options.label_text);
     }
 
     /*uptime monitor options - start*/
-    table = new_frame(global, content,
-                      _(FRAME_TEXT[3]), 1, &global->uptime->enabled);
+    new_monitor_setting(global, GTK_GRID(grid), 10,
+                      _(FRAME_TEXT[3]), &global->uptime->enabled,
+                      NULL, NULL, NULL);
     /*uptime monitor options - end*/
 
     gtk_widget_show_all (dlg);
@@ -879,6 +961,7 @@ monitor_show_about(XfcePanelPlugin *plugin, t_global_monitor *global)
    const gchar *auth[] = {
       "Riccardo Persichetti <riccardo.persichetti@tin.it>",
       "Florian Rivoal <frivoal@xfce.org>",
+      "Landry Breuil <landry@xfce.org>",
       "David Schneider <dnschneid@gmail.com>", NULL };
    icon = xfce_panel_pixbuf_from_source("utilities-system-monitor", NULL, 32);
    gtk_show_about_dialog(NULL,
@@ -888,7 +971,7 @@ monitor_show_about(XfcePanelPlugin *plugin, t_global_monitor *global)
       "program-name", PACKAGE_NAME,
       "comments", _("Monitor CPU load, swap usage and memory footprint"),
       "website", "http://goodies.xfce.org/projects/panel-plugins/xfce4-systemload-plugin",
-      "copyright", _("Copyright (c) 2003-2012\n"),
+      "copyright", _("Copyright (c) 2003-2016\n"),
       "authors", auth, NULL);
 
    if(icon)
@@ -907,15 +990,9 @@ systemload_construct (XfcePanelPlugin *plugin)
     monitor_read_config (plugin, global);
     
     create_monitor (global);
-#ifdef HAS_PANEL_49
     monitor_set_mode (plugin,
                       xfce_panel_plugin_get_mode (plugin),
                       global);
-#else
-    monitor_set_orientation (plugin, 
-                             xfce_panel_plugin_get_orientation (plugin),
-                             global);
-#endif
 
     setup_monitor (global);
 
@@ -925,10 +1002,15 @@ systemload_construct (XfcePanelPlugin *plugin)
 
 #ifdef HAVE_UPOWER_GLIB
     if (global->upower) {
+#if UP_CHECK_VERSION(0, 99, 0)
+        g_signal_connect (global->upower, "notify",
+                          G_CALLBACK(upower_changed_cb), global);
+#else /* UP_CHECK_VERSION < 0.99 */
         g_signal_connect (global->upower, "changed",
                           G_CALLBACK(upower_changed_cb), global);
+#endif /* UP_CHECK_VERSION */
     }
-#endif
+#endif /* HAVE_UPOWER_GLIB */
     
     g_signal_connect (plugin, "free-data", G_CALLBACK (monitor_free), global);
 
@@ -938,13 +1020,16 @@ systemload_construct (XfcePanelPlugin *plugin)
     g_signal_connect (plugin, "size-changed", G_CALLBACK (monitor_set_size),
                       global);
 
-#ifdef HAS_PANEL_49
     g_signal_connect (plugin, "mode-changed",
                       G_CALLBACK (monitor_set_mode), global);
-#else
-    g_signal_connect (plugin, "orientation-changed", 
-                      G_CALLBACK (monitor_set_orientation), global);
-#endif
+
+    g_signal_connect (plugin, "button-press-event", G_CALLBACK (click_event),
+                      global);
+
+    xfce_panel_plugin_menu_insert_item (plugin, GTK_MENU_ITEM (global->menu_item));
+    g_signal_connect (GTK_MENU_ITEM(global->menu_item), "activate",
+                      G_CALLBACK (spawn_system_monitor), global);
+    gtk_widget_set_visible (global->menu_item, global->command.enabled);
 
     xfce_panel_plugin_menu_show_configure (plugin);
     g_signal_connect (plugin, "configure-plugin", 
